@@ -1,7 +1,7 @@
 """Module containing graphQL client."""
 import json
 import logging
-from typing import Callable
+from typing import Any, Callable
 
 import aiohttp
 import requests
@@ -13,10 +13,11 @@ logging.basicConfig(level=logging.INFO, format="%(levelname)s:%(message)s")
 class GraphqlClient:
     """Class which represents the interface to make graphQL requests through."""
 
-    def __init__(self, endpoint: str, headers: dict = None):
+    def __init__(self, endpoint: str, headers: dict = {}, **kwargs: Any):
         """Insantiate the client."""
         self.endpoint = endpoint
-        self.headers = headers or {}
+        self.headers = headers
+        self.options = kwargs
 
     def __request_body(
         self, query: str, variables: dict = None, operation_name: str = None
@@ -31,15 +32,13 @@ class GraphqlClient:
 
         return json
 
-    def __request_headers(self, headers: dict = None) -> dict:
-        return {**self.headers, **headers} if headers else self.headers
-
     def execute(
         self,
         query: str,
         variables: dict = None,
         operation_name: str = None,
-        headers: dict = None,
+        headers: dict = {},
+        **kwargs: Any,
     ):
         """Make synchronous request to graphQL server."""
         request_body = self.__request_body(
@@ -47,7 +46,10 @@ class GraphqlClient:
         )
 
         result = requests.post(
-            self.endpoint, json=request_body, headers=self.__request_headers(headers),
+            self.endpoint,
+            json=request_body,
+            headers={**self.headers, **headers},
+            **{**self.options, **kwargs},
         )
 
         result.raise_for_status()
@@ -58,7 +60,7 @@ class GraphqlClient:
         query: str,
         variables: dict = None,
         operation_name: str = None,
-        headers: dict = None,
+        headers: dict = {},
     ):
         """Make asynchronous request to graphQL server."""
         request_body = self.__request_body(
@@ -69,7 +71,7 @@ class GraphqlClient:
             async with session.post(
                 self.endpoint,
                 json=request_body,
-                headers=self.__request_headers(headers),
+                headers={**self.headers, **headers},
             ) as response:
                 return await response.json()
 
@@ -79,10 +81,13 @@ class GraphqlClient:
         handle: Callable,
         variables: dict = None,
         operation_name: str = None,
-        headers: dict = None,
+        headers: dict = {},
+        init_payload: dict = {},
     ):
         """Make asynchronous request for GraphQL subscription."""
-        connection_init_message = json.dumps({"type": "connection_init", "payload": {}})
+        connection_init_message = json.dumps(
+            {"type": "connection_init", "payload": init_payload}
+        )
 
         request_body = self.__request_body(
             query=query, variables=variables, operation_name=operation_name
@@ -92,7 +97,9 @@ class GraphqlClient:
         )
 
         async with websockets.connect(
-            self.endpoint, subprotocols=["graphql-ws"]
+            self.endpoint,
+            subprotocols=["graphql-ws"],
+            extra_headers={**self.headers, **headers},
         ) as websocket:
             await websocket.send(connection_init_message)
             await websocket.send(request_message)
@@ -100,5 +107,7 @@ class GraphqlClient:
                 response_body = json.loads(response_message)
                 if response_body["type"] == "connection_ack":
                     logging.info("the server accepted the connection")
+                elif response_body["type"] == "ka":
+                    logging.info("the server sent a keep alive message")
                 else:
                     handle(response_body["payload"])
